/********************************************************************** 
 *
 * Description                                                         
 * -----------                                                           
 *  Generate a PNG image from 'netvis.inf', which is the output file of 
 *  the utility 'netvis.pl'. The generated PNG image is saved as
 *  'netvis.png' 
 *
 * invocation:
 *	% netimg 2
 *
 * inputs: 3 files found in current directory
 *	router.png - small icon for router
 *	net.png - small icon for subnet
 *	netvis.dat - description of network as graph
 * output: netvis.png
 *	PNG "image map" of output
 *
 * RCS Keywords
 * ------------
 * $Date: 1999/12/23 17:08:27 $ 
 * $Log: netimg.c,v $
 * Revision 1.2  1999/12/23 17:08:27  root
 * *** empty log message ***
 *
 * Revision 1.1  1999/08/13 03:10:11  bzhang
 * Initial revision
 *
 * $Revision: 1.2 $ 
 *
 * Author
 * ------        
 *   Bin Zhang(bzhang@cs.pdx.edu)
 *
 ***********************************************************************/

/* Bring in gd library functions */
#include "gd.h"
#include "gdfontl.h"
#include "gdfontt.h"
#include "gdfonts.h"
#include "gdfontmb.h"
#include "gdfontg.h"

/* Bring in standard I/O so we can output the image to a file 
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* define constants for small font */
#define FONT_LABEL        gdFontSmall
#define FONT_LABEL_WIDTH  6
#define FONT_LABEL_HEIGHT 12
#define FONT_LABEL_GAP    1

#define MAX_NUM_OF_NODES  100
#define MAX_NUM_OF_LINKS  100
#define MAX_CHAR_PER_LINE 256   

#define OFFSET_X(x)       ((x->nGridSize - x->nRouterW)/2)
#define OFFSET_Y(x)       0
 
#define FN_ROUTER_IMG     "router.png"
#define FN_SUBNET_IMG     "net.png"
#define FN_INPUT          "netvis.dat"
#define FN_OUTPUT         "netvis.png"

/**********************************************
 * Type definitions
 **********************************************/
/* info for converting coordinates from <row, col> to <x,y> */
typedef struct tagCELLDRAW
{
  int nGridSize;
  int nSizeRow;
  int nSizeCol;
  int nRouterW;
  int nRouterH;
  int nNetW;
  int nNetH;
  gdImagePtr im;
  gdImagePtr imR;
  gdImagePtr imN;
}CELL_DRAW;

/* nodes position */
typedef struct tagNODE_POS
{
  char  cFlag;         /* R = router, S = subnet */
  char  strName[64];   /* router's dns name or subnets' ip */
  int   nRow;          /* row */
  int   nCol;          /* column */
}NODE_POS;

/* links */
typedef struct tagLINK_POS
{
	char cFlag;         /* link state */
	int nRow1;
	int nCol1;
	int nRow2;
	int nCol2;
}LINK_POS;


/***********************************************
 * Data Area
 ***********************************************/
int g_numNodes = 0;
NODE_POS g_Nodes[MAX_NUM_OF_NODES]; 

int g_numLinks = 0;
LINK_POS g_Links[MAX_NUM_OF_LINKS];
int g_nAddrLevel = 0; /* display 4 bytes in IP address */

/**************************************
 * function declarations 
 **************************************/
int cdLoadData(CELL_DRAW* pcd, char* pFileName);
int cdLoadRouterImage(CELL_DRAW* pcd, char* pFileName);
int cdLoadNetImage(CELL_DRAW* pcd, char* pFileName);

void cdRectangleAt(CELL_DRAW* pcd, int nRow, int nCol, int color);
void cdLine(CELL_DRAW* pcd, int nRow1, int nCol1, int nRow2, int nCol2, int color);

void cdRouterAt(CELL_DRAW* pcd, int nRow, int nCol, char* pName, int fColor, int bColor);
void cdNetAt(CELL_DRAW* pcd, int nRow, int nCol, char* pName, int fColor, int bColor);

void cdDrawLabel(CELL_DRAW* pcd, int nRow, int nCol, char* pString, int fColor, int bColor);
void cdDrawNodes(CELL_DRAW* pcd);
void cdDrawLinks(CELL_DRAW* pcd);


/***************************************
 *  main function
 ***************************************/
int main(int argc, char* argv[]) 
{
 
        /* Declare an output file */
        FILE *out;
	int grey;

	/* declare the cell draw info structure */
	CELL_DRAW cdInfo = {55, 10,10, 40, 40, 40, 40, NULL, NULL, NULL};
	
	/* process command line */
	if(argc == 2)
	  g_nAddrLevel = atoi(argv[1]);

	printf("netimg: argv[1] %s\n", argv[1]);

	/* load router and subnet image */
	if(cdLoadRouterImage(&cdInfo, FN_ROUTER_IMG) == -1 ||
	   cdLoadNetImage(&cdInfo, FN_SUBNET_IMG) == -1 ||
	   cdLoadData(&cdInfo, FN_INPUT) == -1){
	  printf("Error in loading initial files\n");
	  return 1;
	}

        /* Allocate the image */
        cdInfo.im = gdImageCreate(cdInfo.nSizeCol * cdInfo.nGridSize, 
				  cdInfo.nSizeRow * cdInfo.nGridSize);
        /* setup background */
	grey = gdImageColorAllocate(cdInfo.im, 255, 255, 255);

       	/*  Draw links */
	cdDrawLinks(&cdInfo);
	  
	/*  Draw Routers and subnets */
	cdDrawNodes(&cdInfo);	
	
	
        /* Open a file for writing. */
        out = fopen(FN_OUTPUT, "wb");

        /* Output the image to the disk file. */
        gdImagePng(cdInfo.im, out);

        /* Close the file. */
        fclose(out);

        /* Destroy the image in memory. */
        gdImageDestroy(cdInfo.im);
	gdImageDestroy(cdInfo.imR);
	gdImageDestroy(cdInfo.imN);

	return 0;
}


/******************************************************
 * DESCRIPTION:
 *   Load the image of router symbol
 *
 * PARAMETERS:
 *   CELL_DRAW*        a pointer to drawing structure
 *   char*             file name
 *          
 * RETURN:           
 *   0 = ok, -1 = failed
 ******************************************************/
int cdLoadRouterImage(CELL_DRAW* pcd, char* pFileName)
{
        FILE *in;

	/* load router image */
	in = fopen(pFileName, "rb");
	if(in == NULL)
	  return -1;

	pcd -> imR = gdImageCreateFromPng(in);
	pcd -> nRouterW = pcd -> imR -> sx;
	pcd -> nRouterH = pcd -> imR -> sy;

	fclose(in);
	return 0;
}


/******************************************************
 * DESCRIPTION:
 *   Load the image of subnet symbol
 *
 * PARAMETERS:
 *   CELL_DRAW*        a pointer to drawing structure
 *   char*             file name
 *          
 * RETURN:           
 *   0 = ok, -1 = failed
 ******************************************************/
int cdLoadNetImage(CELL_DRAW* pcd, char* pFileName)
{
        FILE *in;

	/* load router image */
	in = fopen(pFileName, "rb");
	if(in == NULL)
	  return -1;

	pcd -> imN = gdImageCreateFromPng(in);
	pcd -> nNetW = pcd -> imN -> sx;
	pcd -> nNetH = pcd -> imN -> sy;

	fclose(in);
	return 0;
}


/******************************************************
 * DESCRIPTION:
 *   Draw rectangle at specified position
 *
 * PARAMETERS:
 *   CELL_DRAW*        a pointer to drawing structure
 *   int               top
 *   int               left
 *   int               bottom
 *   int               right
 *          
 * RETURN:           
 *   None
 ******************************************************/
void cdRectangleAt(CELL_DRAW* pcd, int nRow, int nCol, int color)
{
     gdImageRectangle(pcd -> im, nCol * pcd -> nGridSize ,  
		      nRow * pcd -> nGridSize,
		      (nCol + 1) * pcd -> nGridSize , 
		      (nRow + 1) * pcd -> nGridSize , color);
}  


/******************************************************
 * DESCRIPTION:
 *   Draw router at specified position
 *
 * PARAMETERS:
 *   CELL_DRAW*        a pointer to drawing structure
 *   int               row
 *   int               column
 *   char*             label
 *   int               foreground color
 *   int               baclground color
 *          
 * RETURN:           
 *   None
 ******************************************************/
void cdRouterAt(CELL_DRAW* pcd, int nRow, int nCol, char* pName, int fColor, int bColor)
{
     gdImageCopy(pcd -> im, pcd -> imR, nCol * pcd -> nGridSize + OFFSET_X(pcd), 
		 nRow * pcd -> nGridSize + OFFSET_Y(pcd), 
		 0, 0, pcd -> nRouterW, pcd -> nRouterH);

     cdDrawLabel(pcd, nRow, nCol, pName, fColor, bColor);
}  


/******************************************************
 * DESCRIPTION:
 *   Draw subnet at specified position
 *
 * PARAMETERS:
 *   CELL_DRAW*        a pointer to drawing structure
 *   int               row
 *   int               column
 *   char*             label
 *   int               foreground color
 *   int               baclground color
 *          
 * RETURN:           
 *   None
 ******************************************************/
void cdNetAt(CELL_DRAW* pcd, int nRow, int nCol, char* pName, int fColor, int bColor)
{
     gdImageCopy(pcd -> im, pcd -> imN, nCol * pcd -> nGridSize + OFFSET_X(pcd), 
		 nRow * pcd -> nGridSize + OFFSET_Y(pcd),
		 0, 0, pcd -> nNetW, pcd -> nNetH);

     cdDrawLabel(pcd, nRow, nCol, pName, fColor, bColor);
}  


/******************************************************
 * DESCRIPTION:
 *   Draw line
 *
 * PARAMETERS:
 *   CELL_DRAW*        a pointer to drawing structure
 *   int               starting row
 *   int               starting column
 *   int               ending row
 *   int               ending column
 *   int               color
 *          
 * RETURN:           
 *   None
 ******************************************************/
void cdLine(CELL_DRAW* pcd, int nRow1, int nCol1, int nRow2, int nCol2, int color)
{
     gdImageLine(pcd -> im, nCol1 * pcd -> nGridSize + pcd -> nGridSize/2 ,  
		      nRow1 * pcd -> nGridSize + pcd -> nGridSize/2,
		      nCol2 * pcd -> nGridSize + pcd -> nGridSize/2 , 
		      nRow2 * pcd -> nGridSize + pcd -> nGridSize/2, color);
}


/******************************************************
 * DESCRIPTION:
 *   Draw label at specified position using specified
 *   foreground and background color
 *
 * PARAMETERS:
 *   CELL_DRAW*        a pointer to drawing structure
 *   int               row
 *   int               column
 *   char*             string of the label
 *   int               foreground color
 *   int               background color
 *          
 * RETURN:           
 *   None
 ******************************************************/
void cdDrawLabel(CELL_DRAW* pcd, int nRow, int nCol, char* pString, int fColor, int bColor)
{
  /* get <x,y> coords in pixels */
  int nX = nCol * pcd -> nGridSize + OFFSET_X(pcd);
  /* leave serveral piexls gap */
  int nY = (nRow + 1) * pcd -> nGridSize - OFFSET_X(pcd)*2 + FONT_LABEL_GAP; 

  int nStrWidth = strlen(pString) * FONT_LABEL_WIDTH;

  /* calcualte the mid-point */
  nX += pcd -> nRouterW / 2;

  /* get x coord of starting point */
  nX -= nStrWidth / 2;
  nX = nX > 0 ? nX : 0;

  /* fill backgroud */
  gdImageFilledRectangle(pcd -> im, nX, nY, nX + nStrWidth, nY + FONT_LABEL_HEIGHT, bColor);

  /* draw label */
  gdImageString(pcd -> im, FONT_LABEL, nX, nY, pString, fColor);
}


/******************************************************
 * DESCRIPTION:
 *   Draw links at the position specified by LINK_POS*,
 *
 * PARAMETERS:
 *   CELL_DRAW*        a pointer to drawing structure
 * 
 * RETURN:           
 *   None
 ******************************************************/
void cdDrawLinks(CELL_DRAW* pcd)
{
  int black, i=0;
  black = gdImageColorAllocate(pcd->im, 0, 0, 0);

  while(i < g_numLinks){
	cdLine(pcd, g_Links[i].nRow1, g_Links[i].nCol1, 
		g_Links[i].nRow2, g_Links[i].nCol2, black);  
	i++;
  }
}


/******************************************************
 * DESCRIPTION:
 *   Draw nodes at the position specified by NODE_POS*,
 *   nFlag = 1 draw routers, nFlag = 0 draw subnets
 *
 * PARAMETERS:
 *   CELL_DRAW*        a pointer to drawing structure
 *
 * RETURN:           
 *   None
 ******************************************************/
void cdDrawNodes(CELL_DRAW* pcd)
{
  int black, white;
  int i=0, nT;
  char* pT; 

  black = gdImageColorAllocate(pcd->im, 0, 0, 0);
  white = gdImageColorAllocate(pcd->im, 255, 255, 255);

  while(i < g_numNodes){
    if(g_Nodes[i].cFlag == 'R'){  
      cdRouterAt(pcd, g_Nodes[i].nRow, g_Nodes[i].nCol, g_Nodes[i].strName, black, white);
    }else if(g_Nodes[i].cFlag == 'S'){
 
      if(g_nAddrLevel < 0 || g_nAddrLevel >= 4)
	g_nAddrLevel = 0;  /* display 4 bytes */

      pT = g_Nodes[i].strName;
      nT = g_nAddrLevel;

      while(nT-- > 0){
	while(*pT++ != '.');
      }

      if(g_nAddrLevel != 0)  
	pT--; /* display the dot sign */

      cdNetAt(pcd, g_Nodes[i].nRow, g_Nodes[i].nCol, pT, black, white);
    }
    i++;	
  }

   
}

/******************************************************
 * DESCRIPTION:
 *   Load drawing data from netvis.inf
 *
 * PARAMETERS:
 *   CELL_DRAW*   a pointer to drawing structure
 *   char*        file name
 *     
 * RETURN:           
 *   0 = ok, -1 = fail
 ******************************************************/
int cdLoadData(CELL_DRAW* pcd, char* pFileName)
{
  FILE* in;
  char  strRawBuf[MAX_CHAR_PER_LINE], cDumy;
  
  in = fopen(pFileName, "r");
  
  if(in == NULL)
	  return -1;

  while(!feof(in)){
    fgets(strRawBuf, MAX_CHAR_PER_LINE, in);
	
	/* R = router S = subnet */
	if(strRawBuf[0] == 'R' || strRawBuf[0] == 'S'){
		if(g_numNodes >= MAX_NUM_OF_NODES)
			continue;

		sscanf(strRawBuf, "%c %s %d %d", 
			&(g_Nodes[g_numNodes].cFlag), g_Nodes[g_numNodes].strName, 
			&(g_Nodes[g_numNodes].nRow), &(g_Nodes[g_numNodes].nCol));

		g_numNodes++;
	}
	/* L = link, U = link is up, D = link is down */
	else if(strRawBuf[0] == 'L' || strRawBuf[0] == 'U' || strRawBuf[0] == 'D'){
		if(g_numLinks >= MAX_NUM_OF_LINKS)
			continue;

		sscanf(strRawBuf, "%c %d %d %d %d", &(g_Links[g_numLinks].cFlag), 
			&(g_Links[g_numLinks].nRow1), &(g_Links[g_numLinks].nCol1), 
			&(g_Links[g_numLinks].nRow2), &(g_Links[g_numLinks].nCol2));

		g_numLinks++;
	/* X = size of the image */
	}else if(strRawBuf[0] == 'X'){
		sscanf(strRawBuf, "%c %d %d", &cDumy, &(pcd -> nSizeRow), &(pcd -> nSizeCol));
	}
		
  }
  
  fclose(in);
  return 0;
}
