#! /usr/bin/perl
########################################################
# neturl.pl - take server parameters and dynamically map them to possibly
#	remote mrtg pages
#
# This is a CGI application.  It only expects an environment variable
#	QUERY_STRING to be set however.
#
# basic idea: impose a level of indirection on the web process.
#
# We take two pieces of info passed to us from the link
# that is clicked on, and determine dynamically some other link to load.
# The CGI program (neturl.pl) is passed two parameters from
# the 2nd-level automatically generated web pages created by
# netvis.html: 1. ip address or DNS name., and 2. a tag.
# The tag indicates a router or subnet page label (top-most link), or
# a router or subnet page sub-entry (LHS links).  There are 4 possible
# kinds of tags in parameter 2:  
# type == (TR, IF, TS, HO)
#   TR=title of router (only for router pages)
#   IF=router interfaces (only for routers)
#   TS=title of subnet (only for subnet pages)
#   HO=subnet hosts
#   SW=switch.html tag can take you somewhere ...
# Router html pages have a label (TR) and LHS links for interfaces (IF).
# Subnet hmtl pages have a label (TS) and LHS links for hosts (HO).
# Clicking on each kind of link (of the 4 kinds) produces the 
# (address, tag) pair. Thus parameter 1 is a unique id, ip address or 
# DNS, that is passed in.
# $address = ip address(IF & HO), router dns(TR), or subnet ID(TS)
#
# The goal of this program is thus to indirectly map (address, tag) into
# normally produced MRTG or other pages (e.g., Big Brother) found elsewhere.  
#
# Associated with the four kinds of tags, we have four lookup tables
# that should be findable with an absolute path via this program
# (see the 4 table variables below).  These 4 variables must be modified below
# for this program to work.
#
# The variable names for the four table files are:
#	$router_label  (for tag TR)
#	$router_if  (for tag IF)
#	$subnet_label  (for tag TS)
#	$subnet_host (for tag HO)
# 
# Here is an example.  Assume we setup the router interface file:
#
#	$router_if = "/www/netvis-data/router_if.txt";
#
# The four lookup tables are loaded according to tag, and searched
# according to value.  Therefore we can map an entry into a remote MRTG page.  
#
# The contents of each table are assumed to be two strings separated by
# white space (blanks ...).  The first label is the ID string (parameter two) 
# passed in.  The # second string is the remote web link that you wish
# to actually display.  The program will map the first into the second,
# and use the HTML "Location" mechanism to redirect the web link.
#
# Here is a very simple example.  Assume you have a router interface 
# with IP address 1.1.1.1.
# Perhaps the desired link for that router interface is a directly
# generated MRTG page:
#
# http://network.some.edu/mrtg/bobrouter.some.edu.5.html
#
# Thus assume that router_if.txt has in it:
#
# 1.1.1.1  http://network.some.edu/mrtg/bobrouter.some.edu.5.html
#
# This program as a CGI-program is passed in two parameters:
#	IP addr or DNS = "bobrouter.some.edu"
#	tag = IF
# Clicking on the 1.1.1.1 LHS link on the router page, will cause
# your web browser to go to the desired MRTG page.
#
# Setup: 
#
# You have to do two things to setup neturl.pl itself.
# 1. setup the four text input tag files.  There is one
# tag file for each type of tag, of course.
# 2. set the four variables below so that neturl.pl can find them.
# 
# Each tag file needs to have a left-hand-side string <white space> and
# a right-hand-side string.  The LHS is the unique address that
# comes in the file.  The RHS is the URL for which you wish to
# redirect the click from the netvis-generated 2nd-level html file.
# Example:
#	in router_label.txt put  (without the perl comment '#"!!!)
#
# bozo-the-router.esu.edu   http://somewhere.edu/bozo-the-router.esu.edu.5.html
#
# Once you setup the four tables, modify the four variables below so
# that neturl.pl (this program) can find those files and search them.
#
# The table format has two features.  Any line that starts with # will
# be ignored (comment).  Any line that starts with * will be treated
# as a wildcard; i.e., no further searching will be performed, and
# that RHS value will be where you go.  This can be used to direct
# all clicks to the same page, or short-circuit the search by putting
# it at the end of an incomplete list.
#
# OTHER NOTIONS:
# lately we have hacked this to work with the rrdtool/14all.cgi app.
# See the code that is commented out below under 14ALLMOD
# If you want to use that in conjunction with 14all, comment in
# and change the url.  The basic idea is that the port and dns name
# are passed to this app, which in turn tunnels into the 14all setup.

# changeable variables #############################
# Set these for the absolute path of the actual lookup tables

$router_label = "/usr/local/apache/cgi-bin/router_label.txt";
$router_if = "/usr/local/apache/cgi-bin/router_if.txt";
$subnet_label = "/usr/local/apache/cgi-bin/subnet_label.txt";
$subnet_host = "/usr/local/apache/cgi-bin/subnet_host.txt";

###################### end changeable variables ###################

@args = split(/\?/, $ENV{QUERY_STRING});

# turn on for debug
#print "Content-type: text/plain\n\n";
#print "tag=$args[0] and address=$args[1] $args[2]\n\n"; 
#exit 0;

$tag = $args[1];
$address = $args[0];
$port=$args[2];
$dns=$args[3];

# based on 2nd parameter, determine file to open

# router page title label (TR)
if ($tag eq 'TR') {
#print "Content-type: text/plain\n\n";
#print "TR: tag is $tag, address is $address\n\n";
#exit;
	$file = $router_label;
}

# router/interface left-hand-side label (ip address)
elsif ($tag eq 'IF') {
# debug - do not remove
#print "Content-type: text/plain\n\n";
#print "args $args[0], $args[1], $args[2], $args[3]\n";
#print "IF: tag is $tag, addr is $address, snmp port = $port\n\n";
#exit;
	$file = $router_if;
### 14ALLMOD
#	$value = "http://bozo.bozo.pdx.edu/cgi-bin/14all.cgi?log=$dns.$port";
#	# local hack
#	print "Location:$value\n\n";
#	exit;
}

# subnet title
elsif ($tag eq 'TS') {
#debug - do not remove
#print "Content-type: text/plain\n\n";
#print "TS: tag is $tag, addr is $address\n\n";
#exit;
	$file = $subnet_label;
}

# subnet/host ip address
elsif ($tag eq 'HO') {
#debug - do not remove
#print "Content-type: text/plain\n\n";
#print "HO: tag is $tag, addr is $address\n\n";
#exit;

	$file = $subnet_host;
}

#switches
#
# note: assumption below is that you have a 14all style redirection.
# this may be replaced with a table as well.
elsif ($tag eq 'SW') {
#debug - do not remove
#print "Content-type: text/plain\n\n";
#print "SW: tag is $tag, addr is $address\n\n";
#exit;
        $value = "http://bozo.bozo.pdx.edu/cgi-bin/14all.cgi?dir=$address";
        print "Location:$value\n\n";
        exit;
}       


# now open the file, read it, and redirect if possible

open(TABLE, $file) || goto fatalpath;   

while(<TABLE>) {
	chop;	# burn newline
	# skip comments
	if (/^#/) {
		next;
	}
	($key, $value) = split(' ', $_, 2);  # split into two parts
	
	# wildcard, i.e., ignore address and just visit where
	# wildcard says to go
	if ($key eq '*') {
 		print "Location:$value\n\n";
		exit;
	}
	$key =~ tr/A-Z/a-z/;	# make sure key is lower-case
	#$table{$key} = $value;	# stuff in associative array
	if ($key eq $address) {
 		print "Location:$value\n\n";
		exit;
	}
	#print "$key, $table{$key}";  # debug printf
	#print "\n";
}
close(TABLE);

notfound:
# no match found, output a default page
print "Content-type: text/plain\n\n";
print "Sorry, there is no information for the host or router link you requested";
exit;

fatalpath:
print "Content-type: text/plain\n\n";
print "fatal error -- there is a flaw in the table path setup\n\n";
exit;
